
""" A list of color constants to use with the ui module """

DEFAULT = 0
BLACK = 1 
RED = 2
GREEN = 3
YELLOW = 4
BLUE = 5
PURPLE = 6
CYAN = 7 
GRAY = 8

BG_BLACK = 16
BG_RED = 32
BG_GREEN = 48 
BG_YELLOW = 64 
BG_BLUE = 80 
BG_PURPLE = 96 
BG_CYAN = 112 
BG_GRAY = 128 

